package com.example.ecgsmartapp;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.UUID;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.InputStreamReader;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.AlertDialog.Builder;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.PopupMenu;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;
import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;


public class MainActivity extends Activity {

	// Message types used by the Handler
	public static final int MESSAGE_WRITE = 1;
	public static final int MESSAGE_READ = 2;

	//Get the default Bluetooth adapter
	BluetoothAdapter bluetooth = BluetoothAdapter.getDefaultAdapter();
	
	/* Create a BroadcastReceiver that will later be used to 
	 receive the names of Bluetooth devices in range. */
	BroadcastReceiver myDiscoverer = new myOwnBroadcastReceiver();

	public BluetoothSocket scSocket;
	
	/* Create a BroadcastReceiver that will later be used to
	 identify if the Bluetooth device is connected */
	BroadcastReceiver checkIsConnected = new myOwnBroadcastReceiver();
	
	private OutputStreamWriter myOutWriter; 
	private File myFile;
	private FileOutputStream fOut;
	private TextView hr_value_TextView;  // hr = heart rate
	private TextView hr_bpm_TextView;
	private LinearLayout layout;
	private int HR=0;  // Heart Rate
	public int flag_start = 0;
	public int flag_stop = 0;	
	public AlertDialog.Builder alertDialogBuilder;
	public int Fs=600;  // Fs is the sampling frequency
	public int delta_plot = 2;  // Fs/delta_plot = sampling frequency used to plot the ECG signal
	
	public Button btnOn; 
	public ToggleButton buttonStart;
	public Button buttonRestart;
	public Button buttonOpen;
	public Button buttonSet;
	public Button button_filter;
	public int index_name_file;
	public EditText edit_name;
	
	public double screenInches;
	public int size_LabelsTextSize;
	public int size_LineWidth;			 
	public int  size_AxisTitleTextSize;
	public int  size_ChartTitleTextSize;
	public int pixel_density;
	
	public byte[] buffer7 = new byte[13];
	public int[] buffer_value_read = new int[8];
	public int control_read_0, control_read_1, control_read_2;
    public byte[] buffer3 = new byte[3];
	public int value_read=0;
	public int check_byte=0;
	
	public int lead=1; 
	public int gain_coeff=1006; // default value; If R5=2.2 kOhm, R8=47 kOhm, R6=220 Ohm, Rp=680 Ohm then GAIN=1006
	public int notch50or60=50; 
	public int saveDataFiltered=0;  // if 0, the data wil be saved unfiltered
	public String files_folder="ECG_Files";  // folder name where ECG files will be saved
	
	private String[] mFileList;
	private File mPath = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + "/" + files_folder);
	private String mChosenFile;
	private static final String FTYPE = ".txt";    
	private static final int DIALOG_LOAD_FILE = 1000;
	
	public String chosen_file;
	public String name_temp;
	public String string_name_file_temp;
	
	public GraphicalView mChart;
    public XYMultipleSeriesDataset dataset=new XYMultipleSeriesDataset();
	public XYMultipleSeriesRenderer mRenderer=new XYMultipleSeriesRenderer();     
	public  XYSeries xSeries=new XYSeries("X Series");
	
	public int control_a=0;
	
	// 3 seconds of the acquired signal are processed and plotted at once
	public int [] buffer_bt= new int[Fs*3];  
	public int [] copy_buffer_bt= new int[Fs*3];
	public int ind_bt=0; 
	public int ready_bt =0;

	public int low_pass_filter_id=40;  // default: Low Pass filter at 40 Hz
	public int high_pass_filter_id=0;  // default: No High Pass filter
	public String plot_title_1="ECG - band: 0.1";
	public String plot_title_2="-40 Hz";
	public String text_lead="LI";
	public String text_notch50or60=" ~ 50 Hz";

	
	// here there is the communication with bluetooth module 
	private final Handler mHandler = new Handler() {
		 @Override
		 public void handleMessage(Message msg) {
		 switch (msg.what) {
		 case MESSAGE_WRITE:
		 //Do something when writing
		 break;
		 case MESSAGE_READ:
		 //Get the bytes from the msg.obj
		 byte[] readBuf = (byte[]) msg.obj;
		 // construct a string from the valid bytes in the buffer
		 

		 for(int u=0; u<msg.arg1; u++) {
		
	
	 // buffer7 has 13 bytes of data from bluetooth that gives 8 values of the ECG signal
	 // such 8 values are divided into two groups (of 4 values) separated by a check byte
	 /* package structure: 
	 
	 1st byte: check byte (=255) 
	 from 2nd to 6th bytes (5 bytes = 40 bits) : 4 values of ECG signal at 10 bit (the 4 values are placed in a row)
	 7th byte: check byte (=255)
	 from 8nd to 12th bytes (5 bytes = 40 bits) : other 4 values of ECG signal at 10 bit (the 4 values are placed in a row)
	 13th byte: check byte (=255)
	 */
	 
			buffer7[12]=buffer7[11];
			buffer7[11]=buffer7[10];
			buffer7[10]=buffer7[9];
			buffer7[9]=buffer7[8];
			buffer7[8]=buffer7[7];
			buffer7[7]=buffer7[6];	
			buffer7[6]=buffer7[5];
		    buffer7[5]=buffer7[4];
		    buffer7[4]=buffer7[3];
		    buffer7[3]=buffer7[2];
		    buffer7[2]=buffer7[1];
			buffer7[1]=buffer7[0];
	        buffer7[0]=readBuf[u];
		 
	        control_read_0=buffer7[12] & 0xFF;
	        control_read_1=buffer7[6] & 0xFF;
	        control_read_2=buffer7[0] & 0xFF;
		
	        if ((control_read_1 == 255) && (control_read_2 == 255) && (control_read_0 == 255))  
		 
	        {	// value_read=((buffer3[1] & 0xFF) << 8)+ (buffer3[0] & 0xFF);

			 if (check_byte==0)
			 {
				buffer_value_read [0] = ((buffer7[10] & 0xC0) >>> 6)+ ((buffer7[11] & 0xFF) << 2);
				buffer_value_read [1] = ((buffer7[9] & 0xF0) >>> 4)+ ((buffer7[10] & 0x3F) << 4);
				buffer_value_read [2] = ((buffer7[8] & 0xFC) >>> 2)+ ((buffer7[9] & 0xF) << 6);
				buffer_value_read [3] = ((buffer7[7] & 0xFF))+ ((buffer7[8] & 0x3) << 8);

				buffer_value_read [4] = ((buffer7[4] & 0xC0) >>> 6)+ ((buffer7[5] & 0xFF) << 2);
				buffer_value_read [5] = ((buffer7[3] & 0xF0) >>> 4)+ ((buffer7[4] & 0x3F) << 4);
				buffer_value_read [6] = ((buffer7[2] & 0xFC) >>> 2)+ ((buffer7[3] & 0xF) << 6);
				buffer_value_read [7] = ((buffer7[1] & 0xFF))+ ((buffer7[2] & 0x3) << 8);		
			 
				if (ind_bt>=Fs*3)
				{
					ind_bt=0;			 
					for (int uu=0; uu<Fs*3; uu++)
						copy_buffer_bt[uu]=buffer_bt[uu];
					
					ready_bt=1;
				}
		 

				buffer_bt[ind_bt]=buffer_value_read [0];
				buffer_bt[ind_bt+1]=buffer_value_read [1];
				buffer_bt[ind_bt+2]=buffer_value_read [2];
				buffer_bt[ind_bt+3]=buffer_value_read [3];
				buffer_bt[ind_bt+4]=buffer_value_read [4];
				buffer_bt[ind_bt+5]=buffer_value_read [5];
				buffer_bt[ind_bt+6]=buffer_value_read [6];
				buffer_bt[ind_bt+7]=buffer_value_read [7];
				ind_bt=ind_bt+8;
		 
				check_byte=1;
		 
		 
			 	}
			 	else
				 check_byte=0;				
	        }	 
		 }
 
		 break;
		 }
		 }
	};
		
	
	
	@Override
	  public void onCreate(Bundle savedInstanceState) {
	    super.onCreate(savedInstanceState);
	  
	    setContentView(R.layout.activity_main);
	    getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);	    
	    hr_value_TextView = (TextView) findViewById(R.id.hr_value_text);
	    buttonStart = (ToggleButton)findViewById(R.id.Button_start);
		buttonRestart = (Button)findViewById(R.id.Button_restart);
		edit_name = (EditText)findViewById(R.id.edit_file_name);
		buttonSet = (Button)findViewById(R.id.Button_set);
		buttonOpen = (Button)findViewById(R.id.Button_open);
		button_filter = (Button)findViewById(R.id.filter);
		hr_bpm_TextView = (TextView)findViewById(R.id.hr_bpm);
		btnOn = (Button) findViewById(R.id.btnOn);
		layout = (LinearLayout) findViewById(R.id.Chart_layout);
		
		buttonStart.setChecked(false);
		buttonRestart.setEnabled(false);
		buttonSet.setEnabled(true);
		buttonStart.setEnabled(false);
		button_filter.setEnabled(false);
		
		index_name_file=0;		
		name_temp = edit_name.getText().toString();
		alertDialogBuilder = new AlertDialog.Builder(this);				

		

		// create ECG examples (txt) and User Manual (pdf) files when the App runs for the first time
		try
		{
				SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
				if(!prefs.getBoolean("firstTime", false))
				{
					
					// copy ECG examples in the ECG_Files folder										    
				    final int[] example_files = new int[] { R.raw.example1, R.raw.example2, R.raw.example3, R.raw.help};				   
				        try {				        			        	
				        	File file_dir = Environment.getExternalStorageDirectory();
						    //add on the your app's path
							File dir = new File(file_dir.getAbsolutePath() + "/" + files_folder);
							//make them in case they're not there
							    dir.mkdirs();
				                Copy_examples(example_files[0], dir + File.separator + "example1.txt");
				                Copy_examples(example_files[1], dir + File.separator + "example2.txt");
				                Copy_examples(example_files[2], dir + File.separator + "example3.txt");
				                Copy_examples(example_files[3], dir + File.separator + "help.pdf");
			                
				        } catch (IOException e) 
				        {}
				    
								
				    SharedPreferences.Editor editor = prefs.edit();
				    editor.putBoolean("firstTime", true);
				    editor.commit();
				}
		}
		catch (Exception e) 
		{ }
		
		
		
			// set title
			alertDialogBuilder.setTitle("ECG device connection");

			// set dialog message
			alertDialogBuilder
				.setMessage("Turn ON the ECG device, then press OK!")
				.setCancelable(false)
				.setPositiveButton("OK",new DialogInterface.OnClickListener() 
				{
					public void onClick(DialogInterface dialog,int id) 
					{
						
						/*IF Bluetooth is NOT enabled, then ask user permission to enable it */
					    if (!bluetooth.isEnabled()) 
					    {
					    Intent requestBluetooth = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
					    startActivityForResult(requestBluetooth, 0); 
					    }
					    else
					    	btnOn.setEnabled(false);
					    
					    
					    /*If Bluetooth is now enabled, then register a broadcastReceiver to report any
					    discovered Bluetooth devices, and then start discovering */
					    if (bluetooth.isEnabled()) 
					    {
					    registerReceiver(myDiscoverer, new IntentFilter(BluetoothDevice.ACTION_FOUND));
					    registerReceiver(checkIsConnected, new IntentFilter(BluetoothDevice.ACTION_ACL_CONNECTED));

					    //Start bluetooth discovery if it is not doing so already
					    	if (!bluetooth.isDiscovering())
					    	{
					    		bluetooth.startDiscovery();
					    		ToastMaster("START DISCOVERY");					
					    	}
					    }						
					}
				  })
				  
				.setNegativeButton("Cancel",new DialogInterface.OnClickListener() 
				{
					public void onClick(DialogInterface dialog,int id) 
					{
						// if this button is clicked, just close
						// the dialog box and do nothing
						dialog.cancel();
						btnOn.setEnabled(true);
					}
				});	
						
	    
		 try {
			 DisplayMetrics dm = new DisplayMetrics();
			 getWindowManager().getDefaultDisplay().getMetrics(dm);
			 int width=dm.widthPixels;
			 int height=dm.heightPixels;
			 pixel_density=dm.densityDpi;
			 double wi=(double)width/(double)pixel_density;
			 double hi=(double)height/(double)pixel_density;
			 double x = Math.pow(wi,2);
			 double y = Math.pow(hi,2);
			 screenInches = Math.sqrt(x+y);  // display dimesion (diagonal) in inch 
			 }
	   
			 catch (Exception e) 
			 {
				 pixel_density=480;			 
			 }
			 
			 
			 if (pixel_density>650)	
			 {
			 size_LabelsTextSize = 35;
			 size_LineWidth = 18;			 
			 size_AxisTitleTextSize = 40;
			 size_ChartTitleTextSize = 45;
			 }
			 else	 
				if (pixel_density<330)	
				{
					 size_LabelsTextSize = 13;
					 size_LineWidth = 5;			 
					 size_AxisTitleTextSize = 13;
					 size_ChartTitleTextSize = 18;
				}
				else
				{
					 size_LabelsTextSize = 25;
					 size_LineWidth = 10;			 
					 size_AxisTitleTextSize = 30;
					 size_ChartTitleTextSize = 35;
				}
	    
	    
	    
	    // read setting file "Setting.txt" if present to import the preferences

	    try 
	    { 
	    
	    	File file_for_settings = getFilesDir ();
			File f_Settings = new File(file_for_settings,"Settings.txt");
			InputStream inputStream = new FileInputStream(f_Settings);
			String read_settings;
	   	 
	        if ( inputStream != null )
	        {
	            InputStreamReader inputStreamReader = new InputStreamReader(inputStream);
	            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
	            String receiveString = "";
	            StringBuilder builder_settings = new StringBuilder("");

	            while((read_settings = bufferedReader.readLine()) != null)
	            {
	            	builder_settings.append(read_settings);
	            }
	            
	            String settings_string = builder_settings.toString();
	            lead= Integer.parseInt(String.valueOf(settings_string.charAt(0)));
	        	int temp_notch50or60=Integer.parseInt(String.valueOf(settings_string.charAt(2)));
	        		if (temp_notch50or60==0)
	        			notch50or60=0;
	        		else
	        			if (temp_notch50or60==6)
	        			notch50or60=60;
	        			else
	        			notch50or60=50;
	        	
	        	saveDataFiltered=Integer.parseInt(String.valueOf(settings_string.charAt(5)));
	        	gain_coeff=Integer.parseInt(settings_string.substring(7)); 
	        	
	        	 if (lead==1)
	        	 {
	        	 text_lead="LI"; 	 	        	 
	        	 }	        	 
		         else
		        	 if (lead==2)
		        	 {
		        		 text_lead="LII";		        		 
		        	 }
		        	 else
		        		 if (lead==3)
		        		 {
		        			 text_lead="LIII";		        			 
			        	 }
		      
		      
	        	 if (notch50or60==50)
		    	  text_notch50or60=" ~ 50 Hz";
		         else
		        	 if (notch50or60==60)
		        		 text_notch50or60=" ~ 60 Hz";
		        	 else
		        		 if (notch50or60==0)
		        			 text_notch50or60=" ~ none";
		      
	            bufferedReader.close();
	            inputStream.close();
	          
	        }
	    }
	   
	     catch (Exception e) {  }
	    

	    
	    Thread thread = new Thread(null, commRunnable, "tag");
		thread.start();
		
	    
	    // to start the recording of ECG
	    buttonStart.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) { 
				
			
				if (buttonStart.isChecked())
				{				
				flag_start = 1;
				buttonOpen.setEnabled(false);
				buttonRestart.setEnabled(false);
				buttonSet.setEnabled(false);
				edit_name.setEnabled(false);
				name_temp = edit_name.getText().toString();
				name_temp = name_temp.replaceAll(" ","");
				
				// an automatic index is generated if more ECG files are recorded in the same session
				index_name_file=index_name_file+1;
				edit_name.setText(name_temp);
				File file_dir = Environment.getExternalStorageDirectory();
			    //add on the your app's path
				File dir = new File(file_dir.getAbsolutePath() + "/" + files_folder);
				//make them in case they're not there
				    dir.mkdirs();
				    //create a standard java.io.File object for the Workbook to use
				    myFile = new File(dir,name_temp+".txt");
				 
				try {
				  	myFile.createNewFile();
			    	fOut = new FileOutputStream(myFile);
			    	myOutWriter = new OutputStreamWriter(fOut);
			    	}
			    	catch (IOException e) {
			    		Toast.makeText(getApplication(),"Impossible to write the file!", Toast.LENGTH_LONG).show();	    					 	
			    	}
			
				}
				
				else
				{
					flag_stop = 1;
					flag_start = 0;
					buttonOpen.setEnabled(true);				
					buttonRestart.setEnabled(true);	
					buttonStart.setEnabled(false);	
					edit_name.setEnabled(true);			
					name_temp = edit_name.getText().toString();
					name_temp = name_temp.replaceAll(" ","");
					
					
					if (name_temp.length()>3)
					{
						if ((name_temp.charAt(name_temp.length()-2)=='_') || ((name_temp.charAt(name_temp.length()-3)=='_') && index_name_file>9))
						{
							if (name_temp.charAt(name_temp.length()-2)=='_')
							{	
							string_name_file_temp=name_temp.substring(0,name_temp.length()-2)+"_"+String.valueOf(index_name_file);	
							edit_name.setText(string_name_file_temp);
							}
							else
							{	
							string_name_file_temp=name_temp.substring(0,name_temp.length()-3)+"_"+String.valueOf(index_name_file);	
							edit_name.setText(string_name_file_temp);
							}
							
						}
						else
						{
						string_name_file_temp=name_temp+"_"+String.valueOf(index_name_file);	
						edit_name.setText(string_name_file_temp);
						}
					}
				
					else
					{
						string_name_file_temp=name_temp+"_"+String.valueOf(index_name_file);	
						edit_name.setText(string_name_file_temp);
					}	
				}
				
			}});
	    
	  
	    
	    buttonRestart.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) 
			{ 
				flag_stop = 0; flag_start = 0; 
				buttonStart.setEnabled(true);
			}});	
	 
	    
		 button_filter.setOnClickListener(new View.OnClickListener()
		 {
		 @Override
		 public void onClick(View v) {

		 PopupMenu popup = new PopupMenu(MainActivity.this, button_filter);  
		 //Inflating the Popup using xml file  
		 popup.getMenuInflater().inflate(R.menu.menu_filter, popup.getMenu());	
		 //registering popup with OnMenuItemClickListener  
		 popup.setOnMenuItemClickListener(new PopupMenu.OnMenuItemClickListener() {  
		  public boolean onMenuItemClick(MenuItem item) { 
			  
			  switch (item.getItemId())
			  {
		        case R.id.filter1:		        	
		        	high_pass_filter_id=0;
		        	low_pass_filter_id=0;		        	
		        	plot_title_1="No filter. ECG - band: 0.1"; 
		        	plot_title_2="-150 Hz";   			        	
		        	break;		     
		        	
		        case R.id.filter2:
		        	high_pass_filter_id=1010;
		        	plot_title_1="ECG - band: 0.1";
		        	Toast.makeText(getApplication(),"Please Wait! High Pass filter needs few seconds to work", Toast.LENGTH_LONG).show();
		        	break;
		        	
		        case R.id.filter3:
		        	high_pass_filter_id=1015;
		        	plot_title_1="ECG - band: 0.15";
		        	Toast.makeText(getApplication(),"Please Wait! High Pass filter needs few seconds to work", Toast.LENGTH_LONG).show();
		        	break;
		        	
		        case R.id.filter4:
		        	high_pass_filter_id=1025;
		        	plot_title_1="ECG - band: 0.25";
		        	Toast.makeText(getApplication(),"Please Wait! High Pass filter needs few seconds to work", Toast.LENGTH_LONG).show();
		        	break;
		        	
		        case R.id.filter5:
		        	high_pass_filter_id=1050;
		        	plot_title_1="ECG - band: 0.5";
		        	Toast.makeText(getApplication(),"Please Wait! High Pass filter needs few seconds to work", Toast.LENGTH_LONG).show();
		        	break;

		        case R.id.filter6:
		        	high_pass_filter_id=1100;
		        	plot_title_1="ECG - band: 1";
		        	Toast.makeText(getApplication(),"Please Wait! High Pass filter needs few seconds to work", Toast.LENGTH_LONG).show();
		        	break;
		        	
		        case R.id.filter7:
		        	low_pass_filter_id=25;		        	
		        	plot_title_2="-25 Hz";
		        	break;
		        	
		        case R.id.filter8:
		        	low_pass_filter_id=35;		        	
		        	plot_title_2="-35 Hz";
		        	break;

		        case R.id.filter9:
		        	low_pass_filter_id=40;		        	
		        	plot_title_2="-40 Hz";
		        	break;

		        case R.id.filter10:
		        	low_pass_filter_id=100;		        	
		        	plot_title_2="-100 Hz";
		        	break;		        	

		        case R.id.filter11:
		        	low_pass_filter_id=150;		        	
		        	plot_title_2="-150 Hz";
		        	break;
		        	
		        case R.id.filter12:
		        	//filter_50_60_Hz_id=50;		        	
		        	//filter_50_60_Hz_TextView.setText("50 Hz removed");
		        	text_notch50or60=" ~ 50 Hz";
		        	notch50or60=50;
		        	break;
		        	
		        case R.id.filter13:
		        	// filter_50_60_Hz_id=60;		        	
		        	// filter_50_60_Hz_TextView.setText("60 Hz removed");
		        	text_notch50or60=" ~ 60 Hz";
		        	notch50or60=60;
		        	break;
		        	
		        case R.id.filter14:
		        	//filter_50_60_Hz_id=0;
		        	notch50or60=0;
		        	text_notch50or60="~ none";
		        	break;
		        	
		        default:
		        	
		        	high_pass_filter_id=0;
		        	low_pass_filter_id=0;
		        	plot_title_1="No filter. ECG - band: 0.1";  
		        	plot_title_2="-150 Hz"; 	
		        	break;
		    }
			   
		   return true;  
		  }  
		 });  
		 popup.show();//showing popup menu
		 }});
	 
		 
	 buttonOpen.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) { 
			buttonOpen.setEnabled(true);
				loadFileList();
				onCreateDialog(1000);				
			}});	
	
	 
	 buttonSet.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) { 
				
			page1();					
			}});	
	
	    
	 btnOn.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) { 
			
				buttonStart.setEnabled(false);
				button_filter.setEnabled(true);
				AlertDialog alertDialog = alertDialogBuilder.create();
				// show it
				alertDialog.show();		
			}});	     
		}
	
	
	
	/*The startActivityForResult() within setup() launches an 
	 Activity which is used to request the user to turn Bluetooth on. 
	 The following onActivityResult() method is called when this 
	 Activity exits. */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		 super.onActivityResult(requestCode, resultCode, data);	
	 if (requestCode==0) {
		 if (resultCode == RESULT_OK) {
			 ToastMaster("Bluetooth has been switched ON");
			 btnOn.setEnabled(false);
			 registerReceiver(myDiscoverer, new IntentFilter(BluetoothDevice.ACTION_FOUND));
			 registerReceiver(checkIsConnected, new IntentFilter(BluetoothDevice.ACTION_ACL_CONNECTED));

			 //Start bluetooth discovery if it is not doing so already
			 if (!bluetooth.isDiscovering()) {
				 bluetooth.startDiscovery();
				 ToastMaster("START DISCOVERY");
			 }
		 } 
		 else {
			 ToastMaster("You need to turn Bluetooth ON !!!");
		 }
	 }
	 
	 
	 // to import the preferences from the settings file
	if (requestCode == 101) {	
				
		lead = data.getIntExtra("lead", lead);  // the second argument is the value to be returned if no value of the desired type is stored with the given name.
		gain_coeff = data.getIntExtra("gain_coeff", gain_coeff);
	    notch50or60 = data.getIntExtra("notch50or60", notch50or60);
	    saveDataFiltered = data.getIntExtra("saveDataFiltered", saveDataFiltered);	
	
	    if (lead==1)
	    	text_lead="LI";
	    else
	    	if (lead==2)
	    		text_lead="LII";
	    	else
	    		if (lead==3)
	    			text_lead="LIII";	      
	      
	      if (notch50or60==50)
	    	  text_notch50or60=" ~ 50 Hz";
	      else
	    	  if (notch50or60==60)
	        		 text_notch50or60=" ~ 60 Hz";
	    	  else
	    		  if (notch50or60==0)
	        			 text_notch50or60=" ~ none";
	         
		}
	    
	}
	
	
	
	
	
	@Override
	public void onPause() {
		super.onPause();
		}
	
	
	 @Override
		public void onDestroy() {
		super.onDestroy();			
		bluetooth.disable();		
		finish();
	    System.exit(0);
		}


	 // load the files list present in the folder ECG_Files
	 public void loadFileList() {
		    try {
		        mPath.mkdirs();
		    }
		    catch(SecurityException e) {
		    }
		    
		    if(mPath.exists()) {
		        FilenameFilter filter = new FilenameFilter() {

		            @Override
		            public boolean accept(File dir, String filename) {
		                File sel = new File(dir, filename);
		                return filename.contains(FTYPE) || sel.isDirectory();
		            }

		        };
		        mFileList = mPath.list(filter);
		    }
		    else {
		        mFileList= new String[0];
		    }    		    
		}

	 
	 
	 // dialog window to chose the ECG file to be opened
		public Dialog onCreateDialog(int id) {
		    Dialog dialog = null;
		    AlertDialog.Builder builder = new Builder(this);

		    switch(id) {
		        case DIALOG_LOAD_FILE:
		            builder.setTitle("Choose your file");
		            if(mFileList == null) {
		                Log.e("lista", "Showing file picker before loading the file list");
		                dialog = builder.create();
		                return dialog;
		            }
		            builder.setItems(mFileList, new DialogInterface.OnClickListener() {
		                public void onClick(DialogInterface dialog, int which) {
		                    mChosenFile = mFileList[which];		                    
		                    chosen_file=mChosenFile;
		    			
		                    page2();
		    
		                }
		            });
		            break;
		    }
		    
		    dialog = builder.show();
		    return dialog;
		}
		
		
		
		private void page2() {
			Intent intent5 = new Intent(this, Pag2.class);
	        intent5.putExtra("scelta", chosen_file);
	        startActivityForResult(intent5, 102);  	
			}

		
		
		private void page1() {
			Intent intent1 = new Intent(this, Pag1.class);
	    
	       intent1.putExtra("lead", lead);
	       intent1.putExtra("gain_coeff", gain_coeff);
	       intent1.putExtra("notch50or60", notch50or60);
	       intent1.putExtra("saveDataFiltered", saveDataFiltered);	   
	       startActivityForResult(intent1, 101);  
			}


		
		// here data are filtered and plotted (a 3 seconds interval is plotted)
		 Runnable commRunnable = new Runnable() {
		    	@Override
		    	public void run() {
		    	int ret = 0;
		    	double [] buffer_plot= new double[Fs*3];
		    	double [] D= {-2,-1,0,1,2};
		    	double output1=0;
		    	double outputd1=0;
		    	double [] buffer_d1= new double[5]; 
		    	double outputd2=0;
		    	double [] buffer_d2= new double[5];
		    	int integer_hr_window = (int)(Fs*2/3);
		    	double [] buffer_hr= new double[integer_hr_window];
		    	int count1=0;
		    	int count2=0;
		    	double summ=0;
		    	double mean1=0;
		    	double mean2=0;
		    	int ind=0;
		    	int read_value=0;
		
		    	double [] filter_hB1= new double[3]; 
		    	double [] filter_hA1= new double[3];
		    	
		    	double [] filter_hB2= new double[3]; 
		    	double [] filter_hA2= new double[3];
		    	
		    	double [] filter_lB1= new double[3]; 
		    	double [] filter_lA1= new double[3];
		    	
		    	double [] filter_lB2= new double[3]; 
		    	double [] filter_lA2= new double[3];
		    		
		    	double [] buffer_fhB1= {0,0,0};
		    	double [] buffer_flB1= {0,0,0};
		    	double [] buffer_fhA1= {0,0,0};
		    	double [] buffer_flA1= {0,0,0};
		    	
		    	double [] buffer_fhB2= {0,0,0};
		    	double [] buffer_flB2= {0,0,0};
		    	double [] buffer_fhA2= {0,0,0};
		    	double [] buffer_flA2= {0,0,0};
		    	
		    	double G1_high=1;
		    	double G2_high=1;
		    	double G1_low=1;
		    	double G2_low=1;
		    	
		    	double output_low1=0;
		    	double output_low2=0;
		    	double output_high1=0;
		    	double output_high2=0;
		    	
		    	double output2_50_60=0;
		    	double output1_50_60=0;
		    	double output3_50_60=0;
		    	double output4_50_60=0;
		    	
		    	 double [] buffer0_50= {0,0,0,0,0,0,0,0,0,0,0,0};
		    	 double [] buffer0_60= {0,0,0,0,0,0,0,0,0,0};
		   	    	
		    	 
		    	 // 50 Hz Notch recursive filter coefficients for Fs= 600 Hz
		    	 	double [] num_filter_50= {1,-1.731688047696030,0.999790560489761};
			    	double [] den_filter_50= {1,-1.695774820284193,0.958550747036629};
			    	double G_filter_50=0.9801104055521309;
			    
			    // 100 Hz Notch recursive filter coefficients for Fs= 600 Hz
			    	double [] num_filter_100= {1,-0.999790560489760,0.999790560489761};
			    	double [] den_filter_100= {1,-0.979056048976068,0.958550747036629};
			    	double G_filter_100=0.9794899854086989; 
			    	
			    // 60 Hz Notch recursive filter coefficients for Fs= 600 Hz
			    	double [] num_filter_60= {1,-1.617695108503741,0.999790560489761};
			    	double [] den_filter_60= {1,-1.584145964134460,0.958550747036629};
			    	double G_filter_60=0.9798734714962504;
			    	
			    // 120 Hz Notch recursive filter coefficients for Fs= 600 Hz
			    	double [] num_filter_120= {1,-0.617904548013980,0.999790560489761};
			    	double [] den_filter_120= {1,-0.605089915158392,0.958550747036629};
			    	double G_filter_120=0.9794336030555506;
		    	 
		    	
		    	double [] buffer_num_filter_50= {0,0,0};
		    	double [] buffer_den_filter_50= {0,0,0};
		    	double [] buffer_num_filter_60= {0,0,0};
		    	double [] buffer_den_filter_60= {0,0,0};
		    	double [] buffer_num_filter_100= {0,0,0};
		    	double [] buffer_den_filter_100= {0,0,0};
		    	double [] buffer_num_filter_120= {0,0,0};
		    	double [] buffer_den_filter_120= {0,0,0};
		    	
		    	double [] buffer2_num_filter_50= {0,0,0};
		    	double [] buffer2_den_filter_50= {0,0,0};
		    	double [] buffer2_num_filter_60= {0,0,0};
		    	double [] buffer2_den_filter_60= {0,0,0};
		    	double [] buffer2_num_filter_100= {0,0,0};
		    	double [] buffer2_den_filter_100= {0,0,0};
		    	double [] buffer2_num_filter_120= {0,0,0};
		    	double [] buffer2_den_filter_120= {0,0,0};
		    	
	    	
		    	// Filter IIR for Fs = 600 Hz
		    	
		    	// The filter is a "Direct Form II Second-Order-Section"
		    	// Order = 4
		    	// Design method: butterworth
		    	// Sections = 2
		    	// Stable: Yes
		    	// Sampling Frequency = 600 Hz
		    	
		    	// "A" identifies the denominator and "B" identifies the numerator
		    	// "1" identifies the first section and "2" identifies the second section
		    	// "G" identifies the gain
		    	
		    	
		// LOW PASS FILTERS
		    	
		    	
		    	// Low Pass , cut off frequency: 25 Hz
		    	//denominators
		    	double [] A1_low25= { 1, -1.757753609482727, 0.819760442927314};
		    	double [] A2_low25= { 1, -1.559054301141692, 0.614051781937882};
		    	// numerators
		    	double [] B1_low25= {1,2,1};
		    	double [] B2_low25= {1,2,1};
		    	// Gains
		    	double G1_low25=0.015501708361147;
		    	double G2_low25=0.013749370199048;
		    	

		    	// Low Pass , cut off frequency: 35 Hz
		    	//denominators
		    	double [] A1_low35= { 1, -1.641977617376592, 0.758796104516994};
		    	double [] A2_low35= { 1, -1.402731984362635, 0.502529342464571};
		    	// numerators
		    	double [] B1_low35= {1,2,1};
		    	double [] B2_low35= {1,2,1};
		    	// Gains
		    	double G1_low35=0.029204621785100;
		    	double G2_low35=0.024949339525484;
		    	
		    	
		    	
		    	// Low Pass , cut off frequency: 40 Hz
		    	//denominators
		    	double [] A1_low40= { 1, -1.581005271466872, 0.730625726656938};
		    	double [] A2_low40= { 1, -1.328044221785862, 0.453725384627135};    	
		    	// numerators
		    	double [] B1_low40= {1,2,1};
		    	double [] B2_low40= {1,2,1};	    	
		    	// Gains
		    	double G1_low40=0.037405113797517;
		    	double G2_low40=0.031420290710318;
		    	
		    	
		    	
		    	// Low Pass , cut off frequency: 100 Hz
		    	//denominators
		    	double [] A1_low100= { 1, -0.751081421656940, 0.502162843313880};
		    	double [] A2_low100= { 1, -0.555523722444109, 0.111047444888218};		    	
		    	// numerators
		    	double [] B1_low100= {1,2,1};
		    	double [] B2_low100= {1,2,1};
		    	// Gains
		    	double G1_low100=0.187770355414235;
		    	double G2_low100=0.138880930611027;
		    	
		    	
		    	// Low Pass , cut off frequency: 150 Hz
		    	//denominators
		    	double [] A1_low150= { 1, -0.160589618511030, 0.446462692171690};
		    	double [] A2_low150= { 1, -0.115415025303165, 0.039566129896580};		    	
		    	// numerators
		    	double [] B1_low150= {1,2,1};
		    	double [] B2_low150= {1,2,1};
		    	// Gains
		    	double G1_low150=0.361615673042922;
		    	double G2_low150=0.259891532474145;
     
		    	
                
	// HIGH PASS FILTERS	
		    	
			    	
		    	// High Pass , cut off frequency: 0.1 Hz
		    	//denominators		    	
		    	double [] A1_high1010= { 1, -1.999197734721228, 0.999198830904549};
		    	double [] A2_high1010= { 1, -1.998065806264844, 0.998066901827515};		    	
		    	// numerators
		    	double [] B1_high1010= {1,-2,1};
		    	double [] B2_high1010= {1,-2,1};
		    	// Gains
		    	double G1_high1010=0.999599141406445;
		    	double G2_high1010=0.999033177023090;
		    	
		    	
		    	// High Pass , cut off frequency: 0.15 Hz
		    	//denominators
		    	double [] A1_high1015= { 1, -1.998796021366643, 0.998798487284930};
		    	double [] A2_high1015= { 1, -1.997099290226235, 0.997101754051262};		    	
		    	// numerators
		    	double [] B1_high1015= {1,-2,1};
		    	double [] B2_high1015= {1,-2,1};
		    	// Gains
		    	double G1_high1015=0.999398627162893;
		    	double G2_high1015=0.998550261069374;
		    	
		    	
		    	
		    	// High Pass , cut off frequency: 0.25 Hz
		    	//denominators	    	
		    	double [] A1_high1025= { 1, -1.997991434939058, 0.997998281967313};
		    	double [] A2_high1025= { 1, -1.995167418322513, 0.995174255672988};		    	
		    	// numerators
		    	double [] B1_high1025= {1,-2,1};
		    	double [] B2_high1025= {1,-2,1};
		    	// Gains
		    	double G1_high1025=0.998997429226593;
		    	double G2_high1025=0.998550261069374;
    

		    	
		    	// High Pass , cut off frequency: 0.5 Hz
		    	//denominators 	
		    	double [] A1_high1050= { 1, -1.995973219799735, 0.996000580481828};
		    	double [] A2_high1050= { 1, -1.990344492411856, 0.990371775935689};		    	
		    	// numerators
		    	double [] B1_high1050= {1,-2,1};
		    	double [] B2_high1050= {1,-2,1};
		    	// Gains
		    	double G1_high1050=0.997993450070391;
		    	double G2_high1050=0.995179067086886;
                    
		    	
		    	// High Pass , cut off frequency: 1 Hz
		    	//denominators		    	
		    	double [] A1_high1100= { 1, -1.991908009819602, 0.992017233388420};
		    	double [] A2_high1100= { 1, -1.980727460109226, 0.980836070607794};		    	
		    	// numerators
		    	double [] B1_high1100= {1,-2,1};
		    	double [] B2_high1100= {1,-2,1};
		    	// Gains
		    	double G1_high1100=0.995981310802006;
		    	double G2_high1100=0.990390882679255;
		    	
		    	 		    	
		    	 XYSeriesRenderer Xrenderer=new XYSeriesRenderer();
		         Xrenderer.setColor(Color.BLUE);
		         Xrenderer.setLineWidth(size_LineWidth);		   
		         mRenderer.setMarginsColor(Color.WHITE);
		         mRenderer.setXLabelsColor(Color.BLACK); 
		         mRenderer.setYLabelsColor(0,Color.BLACK); 
		         mRenderer.setLabelsColor(Color.BLACK);		         		         
		         mRenderer.setChartTitle(plot_title_1+plot_title_2+"  ("+text_lead+")"+text_notch50or60);
		         mRenderer.setXTitle("Time [s] ");		         		         
		         mRenderer.setYTitle("Level");
		         mRenderer.setZoomButtonsVisible(false);
		         mRenderer.setXLabels(0);
		         mRenderer.setPanEnabled(false);		        
		         mRenderer.setClickEnabled(false);
		         mRenderer.addSeriesRenderer(Xrenderer);		         
		         mRenderer.setShowLegend(false);		         
		         mRenderer.setShowGrid(true);
		         
		         
		         //six labels for y-axis in mV
		         //six labels for x-axis in seconds 
		         
		       
		      
			    	int int_label_2 = (int)((Fs/delta_plot)*3/6); 
			    	int int_label_3 = (int)((Fs/delta_plot)*3*2/6); 
			    	int int_label_4 = (int)((Fs/delta_plot)*3*3/6);
			    	int int_label_5 = (int)((Fs/delta_plot)*3*4/6); 
			    	int int_label_6 = (int)((Fs/delta_plot)*3*5/6); 
			    
			    	
		         mRenderer.addXTextLabel(1, String.valueOf(0));
		         mRenderer.addXTextLabel(int_label_2, String.valueOf(0.5));
		         mRenderer.addXTextLabel(int_label_3, String.valueOf(1));
		         mRenderer.addXTextLabel(int_label_4, String.valueOf(1.5));
	          	 mRenderer.addXTextLabel(int_label_5, String.valueOf(2));
		         mRenderer.addXTextLabel(int_label_6, String.valueOf(2.5));
		         mRenderer.addXTextLabel(Fs*3-1, String.valueOf(3));		         
		         Xrenderer.setDisplayChartValues(false);
		         mRenderer.setLabelsTextSize(size_LabelsTextSize);
		         mRenderer.setAxisTitleTextSize(size_AxisTitleTextSize);
		         mRenderer.setChartTitleTextSize(size_ChartTitleTextSize);		      
		         mRenderer.setShowCustomTextGrid(true);		         
		         mRenderer.setYLabels(6);
		         mRenderer.setShowCustomTextGrid(true);
		         mRenderer.setGridColor(Color.BLACK);
		         mRenderer.setYLabelsAngle(270);
				           
		    	while (true) {
		    		

		    		if (ready_bt==1)		    			
		    		{
		    		for (int vv=0; vv<Fs*3; vv++)
		    		{
		    			read_value=copy_buffer_bt[vv]; 
		            switch (low_pass_filter_id) {
		            
		            case 25: // 25 Hz
			   			  
			   			filter_lB1[0]=B1_low25[0];
			   			filter_lB1[1]=B1_low25[1];
			   			filter_lB1[2]=B1_low25[2];

				    	filter_lA1[0]=A1_low25[0];
				    	filter_lA1[1]=A1_low25[1];
				    	filter_lA1[2]=A1_low25[2];

						filter_lB2[0]=B2_low25[0];
			   			filter_lB2[1]=B2_low25[1];
			   			filter_lB2[2]=B2_low25[2];

				    	filter_lA2[0]=A2_low25[0];
				    	filter_lA2[1]=A2_low25[1];
				    	filter_lA2[2]=A2_low25[2];
				    	
				    	G1_low=G1_low25;
				    	G2_low=G2_low25;
			   		 break;
			   		 
			   		 case 35:  // 35 Hz
			   			  
			   			filter_lB1[0]=B1_low35[0];
			   			filter_lB1[1]=B1_low35[1];
			   			filter_lB1[2]=B1_low35[2];
	
				    	filter_lA1[0]=A1_low35[0];
				    	filter_lA1[1]=A1_low35[1];
				    	filter_lA1[2]=A1_low35[2];
	
						filter_lB2[0]=B2_low35[0];
			   			filter_lB2[1]=B2_low35[1];
			   			filter_lB2[2]=B2_low35[2];
	
				    	filter_lA2[0]=A2_low35[0];
				    	filter_lA2[1]=A2_low35[1];
				    	filter_lA2[2]=A2_low35[2];
				    	
				    	G1_low=G1_low35;
				    	G2_low=G2_low35;
			   		 break;
			   		 			   		 
			   		 case 40:  // 40 Hz
			   			 
			   			filter_lB1[0]=B1_low40[0];
			   			filter_lB1[1]=B1_low40[1];
			   			filter_lB1[2]=B1_low40[2];
	
				    	filter_lA1[0]=A1_low40[0];
				    	filter_lA1[1]=A1_low40[1];
				    	filter_lA1[2]=A1_low40[2];
	
						filter_lB2[0]=B2_low40[0];
			   			filter_lB2[1]=B2_low40[1];
			   			filter_lB2[2]=B2_low40[2];
	
				    	filter_lA2[0]=A2_low40[0];
				    	filter_lA2[1]=A2_low40[1];
				    	filter_lA2[2]=A2_low40[2];
				    	
				    	G1_low=G1_low40;
				    	G2_low=G2_low40;
			   			break;
			   			
			   		case 100:  // 100 Hz
			   			
			   			filter_lB1[0]=B1_low100[0];
			   			filter_lB1[1]=B1_low100[1];
			   			filter_lB1[2]=B1_low100[2];
	
				    	filter_lA1[0]=A1_low100[0];
				    	filter_lA1[1]=A1_low100[1];
				    	filter_lA1[2]=A1_low100[2];
	
						filter_lB2[0]=B2_low100[0];
			   			filter_lB2[1]=B2_low100[1];
			   			filter_lB2[2]=B2_low100[2];
	
				    	filter_lA2[0]=A2_low100[0];
				    	filter_lA2[1]=A2_low100[1];
				    	filter_lA2[2]=A2_low100[2];
				    	
				    	G1_low=G1_low100;
				    	G2_low=G2_low100;
			   			break;
			   			
			   		case 150:  // 150 Hz
			   			
			   			filter_lB1[0]=B1_low150[0];
			   			filter_lB1[1]=B1_low150[1];
			   			filter_lB1[2]=B1_low150[2];
	
				    	filter_lA1[0]=A1_low150[0];
				    	filter_lA1[1]=A1_low150[1];
				    	filter_lA1[2]=A1_low150[2];
	
						filter_lB2[0]=B2_low150[0];
			   			filter_lB2[1]=B2_low150[1];
			   			filter_lB2[2]=B2_low150[2];
	
				    	filter_lA2[0]=A2_low150[0];
				    	filter_lA2[1]=A2_low150[1];
				    	filter_lA2[2]=A2_low150[2];
				    	
				    	G1_low=G1_low150;
				    	G2_low=G2_low150;
			   			break;
			   			
			   		default:  // 150 Hz
			   			
			   			filter_lB1[0]=B1_low150[0];
			   			filter_lB1[1]=B1_low150[1];
			   			filter_lB1[2]=B1_low150[2];
	
				    	filter_lA1[0]=A1_low150[0];
				    	filter_lA1[1]=A1_low150[1];
				    	filter_lA1[2]=A1_low150[2];
	
						filter_lB2[0]=B2_low150[0];
			   			filter_lB2[1]=B2_low150[1];
			   			filter_lB2[2]=B2_low150[2];
	
				    	filter_lA2[0]=A2_low150[0];
				    	filter_lA2[1]=A2_low150[1];
				    	filter_lA2[2]=A2_low150[2];
				    	
				    	G1_low=G1_low150;
				    	G2_low=G2_low150;
			   					   			
			            }
			   					           
		            switch (high_pass_filter_id) {		            
		          		            
		            case 1010: // 0.1 Hz
				    	
			   			filter_hB1[0]=B1_high1010[0];
			   			filter_hB1[1]=B1_high1010[1];
			   			filter_hB1[2]=B1_high1010[2];

				    	filter_hA1[0]=A1_high1010[0];
				    	filter_hA1[1]=A1_high1010[1];
				    	filter_hA1[2]=A1_high1010[2];

						filter_hB2[0]=B2_high1010[0];
			   			filter_hB2[1]=B2_high1010[1];
			   			filter_hB2[2]=B2_high1010[2];

				    	filter_hA2[0]=A2_high1010[0];
				    	filter_hA2[1]=A2_high1010[1];
				    	filter_hA2[2]=A2_high1010[2];
				    	
				    	G1_high=G1_high1010;
				    	G2_high=G2_high1010;
			            break;
			            
		            case 1015: // 0.15 Hz
				    	
			   			filter_hB1[0]=B1_high1015[0];
			   			filter_hB1[1]=B1_high1015[1];
			   			filter_hB1[2]=B1_high1015[2];

				    	filter_hA1[0]=A1_high1015[0];
				    	filter_hA1[1]=A1_high1015[1];
				    	filter_hA1[2]=A1_high1015[2];

						filter_hB2[0]=B2_high1015[0];
			   			filter_hB2[1]=B2_high1015[1];
			   			filter_hB2[2]=B2_high1015[2];

				    	filter_hA2[0]=A2_high1015[0];
				    	filter_hA2[1]=A2_high1015[1];
				    	filter_hA2[2]=A2_high1015[2];
				    	
				    	G1_high=G1_high1015;
				    	G2_high=G2_high1015;
			            break;
		            
		            case 1025: // 0.25 Hz
				    	
			   			filter_hB1[0]=B1_high1025[0];
			   			filter_hB1[1]=B1_high1025[1];
			   			filter_hB1[2]=B1_high1025[2];

				    	filter_hA1[0]=A1_high1025[0];
				    	filter_hA1[1]=A1_high1025[1];
				    	filter_hA1[2]=A1_high1025[2];

						filter_hB2[0]=B2_high1025[0];
			   			filter_hB2[1]=B2_high1025[1];
			   			filter_hB2[2]=B2_high1025[2];

				    	filter_hA2[0]=A2_high1025[0];
				    	filter_hA2[1]=A2_high1025[1];
				    	filter_hA2[2]=A2_high1025[2];
				    	
				    	G1_high=G1_high1025;
				    	G2_high=G2_high1025;
			            break;
			            
		            case 1050: // 0.5 Hz
				    	
			   			filter_hB1[0]=B1_high1050[0];
			   			filter_hB1[1]=B1_high1050[1];
			   			filter_hB1[2]=B1_high1050[2];

				    	filter_hA1[0]=A1_high1050[0];
				    	filter_hA1[1]=A1_high1050[1];
				    	filter_hA1[2]=A1_high1050[2];

						filter_hB2[0]=B2_high1050[0];
			   			filter_hB2[1]=B2_high1050[1];
			   			filter_hB2[2]=B2_high1050[2];

				    	filter_hA2[0]=A2_high1050[0];
				    	filter_hA2[1]=A2_high1050[1];
				    	filter_hA2[2]=A2_high1050[2];
				    	
				    	G1_high=G1_high1050;
				    	G2_high=G2_high1050;
			            break;
			            
			   		 case 1100: // 1 Hz
			    	
			   			filter_hB1[0]=B1_high1100[0];
			   			filter_hB1[1]=B1_high1100[1];
			   			filter_hB1[2]=B1_high1100[2];

				    	filter_hA1[0]=A1_high1100[0];
				    	filter_hA1[1]=A1_high1100[1];
				    	filter_hA1[2]=A1_high1100[2];

						filter_hB2[0]=B2_high1100[0];
			   			filter_hB2[1]=B2_high1100[1];
			   			filter_hB2[2]=B2_high1100[2];

				    	filter_hA2[0]=A2_high1100[0];
				    	filter_hA2[1]=A2_high1100[1];
				    	filter_hA2[2]=A2_high1100[2];
				    	
				    	G1_high=G1_high1100;
				    	G2_high=G2_high1100;	
		            break;
			   		  		                     		            
		            default:  // 0.1 HZ
		            	
		            	filter_hB1[0]=B1_high1100[0];
			   			filter_hB1[1]=B1_high1100[1];
			   			filter_hB1[2]=B1_high1100[2];

				    	filter_hA1[0]=A1_high1100[0];
				    	filter_hA1[1]=A1_high1100[1];
				    	filter_hA1[2]=A1_high1100[2];

						filter_hB2[0]=B2_high1100[0];
			   			filter_hB2[1]=B2_high1100[1];
			   			filter_hB2[2]=B2_high1100[2];

				    	filter_hA2[0]=A2_high1100[0];
				    	filter_hA2[1]=A2_high1100[1];
				    	filter_hA2[2]=A2_high1100[2];
				    	
				    	G1_high=G1_high1100;
				    	G2_high=G2_high1100;
		            	
		            }			    	
		            
		            
		           // 50 or 60 Hz removal filter		            
		            
		            if (notch50or60==50)  // notch at 50 Hz
		        	{		
		            	
		            	if ((low_pass_filter_id==35) || (low_pass_filter_id==25))  // if a Low pass filter is selected at 35 or 25 Hz
		            		
		            	{ // Moving average filter with zeros at 50 , 100 , 150 , ... Hz (it is a notch filter + low pass filter at about 35 Hz
		            		buffer0_50[11]=buffer0_50[10];
		            		buffer0_50[10]=buffer0_50[9];
		            		buffer0_50[9]=buffer0_50[8];
		            		buffer0_50[8]=buffer0_50[7];
		            		buffer0_50[7]=buffer0_50[6];
		            		buffer0_50[6]=buffer0_50[5];
		            		buffer0_50[5]=buffer0_50[4];
			            	buffer0_50[4]=buffer0_50[3];
			            	buffer0_50[3]=buffer0_50[2];
							buffer0_50[2]=buffer0_50[1];
					        buffer0_50[1]=buffer0_50[0];
					        buffer0_50[0]=read_value;         				    					  
					          
					        output4_50_60=((buffer0_50[0]+buffer0_50[1]+buffer0_50[2]+buffer0_50[3]+buffer0_50[4]+buffer0_50[5]+buffer0_50[6]+buffer0_50[7]+buffer0_50[8]+buffer0_50[9]+buffer0_50[10]+buffer0_50[11])/12);
					      				          
		            	}
		            	else
		            	{
				          // Notch recursive filter at 50 Hz is applied twice
				          
		            		buffer_num_filter_50[2]=buffer_num_filter_50[1];
		            		buffer_num_filter_50[1]=buffer_num_filter_50[0];
		            		buffer_num_filter_50[0]=read_value;         
		            		buffer_den_filter_50[2]=buffer_den_filter_50[1];
		            		buffer_den_filter_50[1]=output1_50_60;						  
		            		output1_50_60=G_filter_50*(buffer_num_filter_50[0]*num_filter_50[0]+buffer_num_filter_50[1]*num_filter_50[1]+buffer_num_filter_50[2]*num_filter_50[2])-buffer_den_filter_50[1]*den_filter_50[1]-buffer_den_filter_50[2]*den_filter_50[2];
				 		  				          
		            		buffer2_num_filter_50[2]=buffer2_num_filter_50[1];
		            		buffer2_num_filter_50[1]=buffer2_num_filter_50[0];
		            		buffer2_num_filter_50[0]=output1_50_60;         
		            		buffer2_den_filter_50[2]=buffer2_den_filter_50[1];
		            		buffer2_den_filter_50[1]=output2_50_60;						  
		            		output2_50_60=G_filter_50*(buffer2_num_filter_50[0]*num_filter_50[0]+buffer2_num_filter_50[1]*num_filter_50[1]+buffer2_num_filter_50[2]*num_filter_50[2])-buffer2_den_filter_50[1]*den_filter_50[1]-buffer2_den_filter_50[2]*den_filter_50[2];
				 		   				          
				          
		            		// Notch recursive filter at 100 Hz is applied twice  
				          
					        buffer_num_filter_100[2]=buffer_num_filter_100[1];
					        buffer_num_filter_100[1]=buffer_num_filter_100[0];
					        buffer_num_filter_100[0]=output2_50_60;         
					        buffer_den_filter_100[2]=buffer_den_filter_100[1];
					        buffer_den_filter_100[1]=output3_50_60;							  
					        output3_50_60=G_filter_100*(buffer_num_filter_100[0]*num_filter_100[0]+buffer_num_filter_100[1]*num_filter_100[1]+buffer_num_filter_100[2]*num_filter_100[2])-buffer_den_filter_100[1]*den_filter_100[1]-buffer_den_filter_100[2]*den_filter_100[2];				 		 
						
					        buffer2_num_filter_100[2]=buffer2_num_filter_100[1];
					        buffer2_num_filter_100[1]=buffer2_num_filter_100[0];
					        buffer2_num_filter_100[0]=output3_50_60;         
					        buffer2_den_filter_100[2]=buffer2_den_filter_100[1];
					        buffer2_den_filter_100[1]=output4_50_60;							  
					        output4_50_60=G_filter_100*(buffer2_num_filter_100[0]*num_filter_100[0]+buffer2_num_filter_100[1]*num_filter_100[1]+buffer2_num_filter_100[2]*num_filter_100[2])-buffer2_den_filter_100[1]*den_filter_100[1]-buffer2_den_filter_100[2]*den_filter_100[2];				 		 						     
		            	}
				          
		                            
				    }
		        	else
		        		 if (notch50or60==60)  // notch at 60 Hz
		        		 {		
		            	  
		        			if ((low_pass_filter_id==35) || (low_pass_filter_id==25))  // if a Low pass filter is selected at 35 or 25 Hz
		        			
		        			{ // Moving average filter with zeros at 60 , 120 , 180 , ... Hz (it is a notch filter + low pass filter at about 35 Hz
				            				            	   		        					
			        		buffer0_60[9]=buffer0_60[8];
			        		buffer0_60[8]=buffer0_60[7];
			        		buffer0_60[7]=buffer0_60[6];
			        		buffer0_60[6]=buffer0_60[5];
			        		buffer0_60[5]=buffer0_60[4];
			        		buffer0_60[4]=buffer0_60[3];
				            buffer0_60[3]=buffer0_60[2];
							buffer0_60[2]=buffer0_60[1];
					        buffer0_60[1]=buffer0_60[0];
					        buffer0_60[0]=read_value;         					    					  
					        output4_50_60=((buffer0_60[0]+buffer0_60[1]+buffer0_60[2]+buffer0_60[3]+buffer0_60[4]+buffer0_60[5]+buffer0_60[6]+buffer0_60[7]+buffer0_60[8]+buffer0_60[9])/10);			
				                 
		        			}
		        			else
		        			{
		        				// Notch recursive filter at 60 Hz is applied twice  
				          
		        		      buffer_num_filter_60[2]=buffer_num_filter_60[1];
					          buffer_num_filter_60[1]=buffer_num_filter_60[0];
					          buffer_num_filter_60[0]=read_value;         
					          buffer_den_filter_60[2]=buffer_den_filter_60[1];
					          buffer_den_filter_60[1]=output1_50_60;							  
					          output1_50_60=G_filter_60*(buffer_num_filter_60[0]*num_filter_60[0]+buffer_num_filter_60[1]*num_filter_60[1]+buffer_num_filter_60[2]*num_filter_60[2])-buffer_den_filter_60[1]*den_filter_60[1]-buffer_den_filter_60[2]*den_filter_60[2];
					 		  					          
					          buffer2_num_filter_60[2]=buffer2_num_filter_60[1];
					          buffer2_num_filter_60[1]=buffer2_num_filter_60[0];
					          buffer2_num_filter_60[0]=output1_50_60;         
					          buffer2_den_filter_60[2]=buffer2_den_filter_60[1];
					          buffer2_den_filter_60[1]=output2_50_60;							  
					          output2_50_60=G_filter_60*(buffer2_num_filter_60[0]*num_filter_60[0]+buffer2_num_filter_60[1]*num_filter_60[1]+buffer2_num_filter_60[2]*num_filter_60[2])-buffer2_den_filter_60[1]*den_filter_60[1]-buffer2_den_filter_60[2]*den_filter_60[2];
					 		   
					          
					          
					       // Notch recursive filter at 120 Hz is applied twice        
					 		          
					          buffer_num_filter_120[2]=buffer_num_filter_120[1];
					          buffer_num_filter_120[1]=buffer_num_filter_120[0];
					          buffer_num_filter_120[0]=output2_50_60;         
					          buffer_den_filter_120[2]=buffer_den_filter_120[1];
					          buffer_den_filter_120[1]=output3_50_60;							  
					          output3_50_60=G_filter_120*(buffer_num_filter_120[0]*num_filter_120[0]+buffer_num_filter_120[1]*num_filter_120[1]+buffer_num_filter_120[2]*num_filter_120[2])-buffer_den_filter_120[1]*den_filter_120[1]-buffer_den_filter_120[2]*den_filter_120[2];				 		 
						
					          buffer2_num_filter_120[2]=buffer2_num_filter_120[1];
					          buffer2_num_filter_120[1]=buffer2_num_filter_120[0];
					          buffer2_num_filter_120[0]=output3_50_60;         
					          buffer2_den_filter_120[2]=buffer2_den_filter_120[1];
					          buffer2_den_filter_120[1]=output4_50_60;							  
					          output4_50_60=G_filter_120*(buffer2_num_filter_120[0]*num_filter_120[0]+buffer2_num_filter_120[1]*num_filter_120[1]+buffer2_num_filter_120[2]*num_filter_120[2])-buffer2_den_filter_120[1]*den_filter_120[1]-buffer2_den_filter_120[2]*den_filter_120[2];				 		 
		        			}
		        			
		        		 }
		        		 else
		        		  output4_50_60=read_value;  // no noth fitler applied
		           
		            
		        	
		            //low pass filter
		            if (low_pass_filter_id!=0)
		        	{			      				
						  buffer_flB1[2]=buffer_flB1[1];
				          buffer_flB1[1]=buffer_flB1[0];
				          buffer_flB1[0]=output4_50_60;         
				          buffer_flA1[2]=buffer_flA1[1];
				          buffer_flA1[1]=output_low1;						  
				          output_low1=G1_low*(buffer_flB1[0]*filter_lB1[0]+buffer_flB1[1]*filter_lB1[1]+buffer_flB1[2]*filter_lB1[2])-buffer_flA1[1]*filter_lA1[1]-buffer_flA1[2]*filter_lA1[2];
				 		          
				          buffer_flB2[2]=buffer_flB2[1];
				          buffer_flB2[1]=buffer_flB2[0];
				          buffer_flB2[0]=output_low1;         
				          buffer_flA2[2]=buffer_flA2[1];
				          buffer_flA2[1]=output_low2;						  
				          output_low2=G2_low*(buffer_flB2[0]*filter_lB2[0]+buffer_flB2[1]*filter_lB2[1]+buffer_flB2[2]*filter_lB2[2])-buffer_flA2[1]*filter_lA2[1]-buffer_flA2[2]*filter_lA2[2];				 		 
					 }
		        	else
		        		output_low2=output4_50_60;
		           
		            
		            //high pass filter
		        	if (high_pass_filter_id!=0)
			        {	
		                  buffer_fhB1[2]=buffer_fhB1[1];
		                  buffer_fhB1[1]=buffer_fhB1[0];
		                  buffer_fhB1[0]=output_low2;		                  
		                  buffer_fhA1[2]=buffer_fhA1[1];
		                  buffer_fhA1[1]=output_high1;		        		  
		                  output_high1=G1_high*(buffer_fhB1[0]*filter_hB1[0]+buffer_fhB1[1]*filter_hB1[1]+buffer_fhB1[2]*filter_hB1[2])-buffer_fhA1[1]*filter_hA1[1]-buffer_fhA1[2]*filter_hA1[2];
		         		 		                  
		                  buffer_fhB2[2]=buffer_fhB2[1];
		                  buffer_fhB2[1]=buffer_fhB2[0];
		                  buffer_fhB2[0]=output_high1;		                  
		                  buffer_fhA2[2]=buffer_fhA2[1];
		                  buffer_fhA2[1]=output_high2;		        		  
		                  output_high2=G2_high*(buffer_fhB2[0]*filter_hB2[0]+buffer_fhB2[1]*filter_hB2[1]+buffer_fhB2[2]*filter_hB2[2])-buffer_fhA2[1]*filter_hA2[1]-buffer_fhA2[2]*filter_hA2[2];      		
			        }
		        	else
		        		output_high2=output_low2;
		        	
		        	
          output1=output_high2;
 
          
          // save ECG data into a file (from START to STOP)
          if (flag_start == 1) 
          {	
		        	 	
		    	   try {		        					        			
		    		   double value_to_write=read_value;		        			
		        			if (saveDataFiltered == 0) 	// save data unfiltered
		        				myOutWriter.append(String.valueOf((value_to_write*5/1023)*1000/gain_coeff));
		        			else // save data filtered
		        				myOutWriter.append(String.valueOf((output1*5/1023)*1000/gain_coeff));
		        		 
		        			
		        			myOutWriter.append(System.getProperty("line.separator"));
		        			
		        		} catch (IOException e) {}
		        		
          }
		            
		            
		         // Heart Rate Calculation
		            
		            buffer_plot[ind]=output1;

		            buffer_d1[4]=buffer_d1[3];
		    		buffer_d1[3]=buffer_d1[2];
		            buffer_d1[2]=buffer_d1[1];
		    		buffer_d1[1]=buffer_d1[0];
		            buffer_d1[0]=output1;		    		
		            outputd1=(buffer_d1[0]*D[0]+buffer_d1[1]*D[1]+buffer_d1[2]*D[2]+buffer_d1[3]*D[3]+buffer_d1[4]*D[4]);
		            
		            buffer_d2[4]=buffer_d2[3];
		    		buffer_d2[3]=buffer_d2[2];
		            buffer_d2[2]=buffer_d2[1];
		    		buffer_d2[1]=buffer_d2[0];
		            buffer_d2[0]=outputd1;		    		
		            outputd2=(buffer_d2[0]*D[0]+buffer_d2[1]*D[1]+buffer_d2[2]*D[2]+buffer_d2[3]*D[3]+buffer_d2[4]*D[4]);
		            
		            count1=count1+1;
		            count2=count2+1;
		            		        
		            int integer_hr_window_minus1=integer_hr_window-1;
		            
		            for (int y = 0; y < integer_hr_window_minus1; y++) {
					buffer_hr[integer_hr_window_minus1-y]=buffer_hr[integer_hr_window_minus1-1-y];
			    	}		 
		            
		            buffer_hr[0]=outputd2;		            
		            summ=0;
		            
		            for (int y = 0; y < integer_hr_window_minus1; y++) {
		    			summ=summ+Math.abs(buffer_hr[y]);
		    	    	}
		            		            
		            mean1=summ*2/integer_hr_window;
		            mean2=mean1*2;		            
		            int hr_window_half = (int)(integer_hr_window/2); 		            
		            int hr_40ms = (int)(0.04*Fs); 
		            
		            if ((-buffer_hr[hr_window_half]>mean2)  
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half-10] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half-9]  
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half-8] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half-7]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half-6] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half-5]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half-4] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half-3]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half-2] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half-1]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half+1] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half+2]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half+3] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half+4]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half+5] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half+6]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half+7] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half+8]
		            		&& buffer_hr[hr_window_half]<buffer_hr[hr_window_half+9] && buffer_hr[hr_window_half]<buffer_hr[hr_window_half+10])
		            	count1=0;
		            
		            if ((buffer_hr[hr_window_half]>mean1)  
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half-10] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half-9]  
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half-8] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half-7]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half-6] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half-5]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half-4] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half-3]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half-2] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half-1]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half+1] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half+2]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half+3] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half+4]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half+5] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half+6]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half+7] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half+8]
		            		&& buffer_hr[hr_window_half]>buffer_hr[hr_window_half+9] && buffer_hr[hr_window_half]>buffer_hr[hr_window_half+10])
		            	{ 
		            		if (count1<hr_40ms)
		            		{
		            			HR=Fs*60/count2;
		            			count2=0;		         
		            		}
		            		else{}
		            	}
		            
		          		            		            
		            ind=ind+1;

		    		if (ind == Fs*3)
		    		{
		    			ind = 0;		    	
		    			 xSeries.clear();		    			 
		    			 int ind_plot=1;
		    			 int ind_buffer_plot=1;
		    			 
		    			 while (ind_buffer_plot<buffer_plot.length)
		    			 {		    				 
		    			  xSeries.add(ind_plot,buffer_plot[ind_buffer_plot]);
		    		      ind_plot=ind_plot+1;
		    		      ind_buffer_plot=ind_buffer_plot+delta_plot;
		    			 }

		    			 
		    	       // plot data
				    	runOnUiThread(new Runnable() {
				    	@Override
				    	public void run() {
				    		 
	
				    		 if (flag_stop == 0 && control_a==0) {				    			 		    		
				    			 mRenderer.setChartTitle(plot_title_1+plot_title_2+"  ("+text_lead+")"+text_notch50or60);
				    			 dataset.addSeries(xSeries);    			 
				    			 mChart=(GraphicalView)ChartFactory.getLineChartView(getBaseContext(), dataset, mRenderer);   			
				        	     layout.addView(mChart);	
				        	     control_a=1;			        	     
				    		 }
				   		 
				    		 
				    		 if (flag_stop == 0 && control_a==1) {	
				    			 dataset.clear();
				    			 mRenderer.setChartTitle(plot_title_1+plot_title_2+"  ("+text_lead+")"+text_notch50or60);
				    			 dataset.addSeries(xSeries);
				        	     mChart.repaint();	        	     		        	     
				    		 }	  
				    		 			    		 
				    		 String s_bpm = String.valueOf(HR);	
				    		 
				    		 if (lead==1)
				    		 {
				    			 hr_value_TextView.setText(s_bpm);
				    			 hr_bpm_TextView.setText("  bpm  ");
				    		 }
				    		 else 
				    		 {	 hr_value_TextView.setText("only for Lead I");
				    		   	 hr_bpm_TextView.setText("");
				    		 }
				    		
				    	}  // end run			    	
				    	}); // end runOnUiThread
	    	
		    		} // end if
		    		} // end for
		    				    		
		    		ready_bt=0;
		    	
		    		} // end if
		    				    	
		    	} // end while		    	
	    	
		    	} // end run
		  	};  // end commRunnable
		    	
		    	
	 
	/* This BroadcastReceiver will display discovered Bluetooth devices */
	public class myOwnBroadcastReceiver extends BroadcastReceiver {
		 ConnectToBluetooth connectBT;
	
		 @Override
		 public void onReceive(Context context, Intent intent) {
		 String action=intent.getAction();
	
		 //Notification that BluetoothDevice is FOUND
		 if (BluetoothDevice.ACTION_FOUND.equals(action)) {
		 //Display the name of the discovered device
		 String discoveredDeviceName = intent.getStringExtra(BluetoothDevice.EXTRA_NAME);
		 ToastMaster("Discovered: " + discoveredDeviceName);
	
		 //Display more information about the discovered device
		 BluetoothDevice discoveredDevice = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
	
		 int bondyState=discoveredDevice.getBondState();
	
		 String mybondState;
		 switch(bondyState) {
		 case 10: 
		 mybondState="BOND_NONE";
		 break;
		 case 11: 
		 mybondState="BOND_BONDING";
		 break;
		 case 12: 
		 mybondState="BOND_BONDED";
		 break;
		 default: 
		 mybondState="INVALID BOND STATE";
		 break;
		 }
		
			 if(discoveredDeviceName!=null)
			 {
				 //Connect to the discovered bluetooth device (SeeedBTSlave)
				 if (discoveredDeviceName.equals("HC-06") || discoveredDeviceName.equals("HC-05")) {
				 unregisterReceiver(myDiscoverer);
				 connectBT = new ConnectToBluetooth(discoveredDevice);
				 //Connect to the the device in a new thread
				 new Thread(connectBT).start();
				 }
			 }
		 }
	
		 //Notification if bluetooth device is connected
		 if (BluetoothDevice.ACTION_ACL_CONNECTED.equals(action)) {
		 while (scSocket==null) {
		 //do nothing
		 }
		 ToastMaster("CONNECTED");
		 buttonStart.setEnabled(true);
		 
		// BTisConnected=true; //turn screen purple 
		 if (scSocket!=null) {
		 SendReceiveBytes sendReceiveBT = new SendReceiveBytes(scSocket);
		 new Thread(sendReceiveBT).start();
		 		
		 }
		 }
		 }
	} // end class myOwnBroadcastReceiver
	
	
	public class ConnectToBluetooth implements Runnable {
			 private BluetoothDevice btShield;
			 private BluetoothSocket mySocket = null;
			 private UUID uuid = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
			 
			 public ConnectToBluetooth(BluetoothDevice bluetoothShield) {
				 btShield = bluetoothShield;			 
				 try {
				 mySocket = btShield.createRfcommSocketToServiceRecord(uuid);
				 }
				 catch(IOException createSocketException) {
				 //Problem with creating a socket
				 Log.e("ConnectToBluetooth", "Error with Socket");
				 }			
					 /* Cancel discovery on Bluetooth Adapter to prevent slow connection */
					 bluetooth.cancelDiscovery();
					 ToastMaster("stop discovey");					
			 }
	
			 
			 @Override
			 public void run() {			 
				 
				 try {				 
				 /*Connect to the bluetoothShield through the Socket. This will block
				 until it succeeds or throws an IOException */
				 mySocket.connect();
				 scSocket=mySocket;
				 } 
				 catch (IOException connectException) {
					 Log.e("ConnectToBluetooth", "Error with Socket Connection");
					 
					 try {
					 mySocket.close(); //try to close the socket
					 }
					 catch(IOException closeException) {
					 }
					 return;
				 }
			 }
	
			 /* Will cancel an in-progress connection, and close the socket */
			 public void cancel() {
				 try {
				 mySocket.close();
				 } 
				 catch (IOException e) {
				 }
			 }
			 
		} // end class ConnectToBluetooth
	
	
	private class SendReceiveBytes implements Runnable {
		 private BluetoothSocket btSocket;
		 private InputStream btInputStream = null;
		 private OutputStream btOutputStream = null;
		 String TAG = "SendReceiveBytes";

		 public SendReceiveBytes(BluetoothSocket socket) {
			 btSocket = socket;
			 try {
			 btInputStream = btSocket.getInputStream();
			 btOutputStream = btSocket.getOutputStream();
			 } 
			 catch (IOException streamError) { 
			 Log.e(TAG, "Error when getting input or output Stream");
			 }
		 }

		 public void run() {
		 
			 int bytes; // bytes returned from read()
	
			 // Keep listening to the InputStream until an exception occurs
			 while (true) {
				 byte[] buffer = new byte[200]; // buffer store for the stream
				 try {
				 // Read from the InputStream
				 bytes = btInputStream.read(buffer);
				 // Send the obtained bytes to the UI activity
				 mHandler.obtainMessage(MESSAGE_READ, bytes, -1, buffer)
				 .sendToTarget();
				 } 
				 catch (IOException e) {
				 Log.e(TAG, "Error reading from btInputStream");
				 break;
				 }
			 }
		 }

		 /* Call this from the main activity to send data to the remote device */
		 public void write(byte[] bytes) {
			 try {
			 btOutputStream.write(bytes);
			 } 
			 catch (IOException e) { 
			 Log.e(TAG, "Error when writing to btOutputStream");
			 }
		 }


		 /* Call this from the main activity to shutdown the connection */
		 public void cancel() {
			 try {
			 btSocket.close();
			 } 
			 catch (IOException e) { 
			 Log.e(TAG, "Error when closing the btSocket");
			 }
			 }
		 
		}  // end  class SendReceiveBytes 


		/* My ToastMaster function to display a messageBox on the screen */
		void ToastMaster(String textToDisplay) {
			 Toast myMessage = Toast.makeText(getApplicationContext(), 
			 textToDisplay, 
			 Toast.LENGTH_SHORT);
			 myMessage.setGravity(Gravity.CENTER, 0, 0);
			 myMessage.show();
		}
		
		
		// funciton to copy ecg example files from the raw folder
		public void Copy_examples(int id, String path) throws IOException {
		    InputStream in = getResources().openRawResource(id);
		    FileOutputStream out = new FileOutputStream(path);
		    byte[] buff = new byte[1024];
		    int read = 0;
		    try {
		        while ((read = in.read(buff)) > 0) {
		            out.write(buff, 0, read);
		        }
		    } finally {
		        in.close();
		        out.close();
		    }
		}
		
		
}  // end class MainActivity


